#!/usr/bin/env perl

use strict;
use warnings;
use Test::More tests => 4;
use ok 'Catalyst::View::Download';
use ok 'Catalyst::View::Download::CSV';
use ok 'Catalyst::View::Download::Plain';
use ok 'Catalyst::View::Download::HTML';
